*------------------------------------------------------------------------*
*   This file includes all the necessary routine for 3Body integration   *
*   of cross sections or widths                                          *
*                                                                        *
*   Author: Jorge C. Romao                                               *
*   Date: 16.03.2022                                                     *
*   Email: jorge.romao@tecnico.ulisboa.pt                                *
*------------------------------------------------------------------------*

      
*------------------------------------------------------------------------*
      subroutine sigma(xs,method,npoints)
      implicit none
      
      
********************************************************************
*                                                                  *      
***   Start variables for Integration in Cuba 4.2                  *
*                                                                  *
********************************************************************
*
***   Variables for Gauss
*

      integer npoints,method
*
***   Variables for all the methods
*
	integer ndim, ncomp, nvec, last, seed, mineval, maxeval
	real*8 epsrel, epsabs, userdata
	parameter (ndim = 4)
	parameter (ncomp = 1)
	parameter (userdata = 0)
	parameter (nvec = 1)
	parameter (epsrel = 1D-3)
	parameter (epsabs = 1D-12)
	parameter (last = 4)
	parameter (seed = 0)
	parameter (mineval = 0)
	parameter (maxeval = 50000)

	integer nstart, nincrease, nbatch, gridno
	integer*8 spin
	character*(*) statefile	
	parameter (nstart = 1000)
	parameter (nincrease = 500)
	parameter (nbatch = 1000)
	parameter (gridno = 0)
	parameter (statefile = "")
	parameter (spin = -1)

	integer nnew, nmin
	real*8 flatness
	parameter (nnew = 1000)
	parameter (nmin = 2)
	parameter (flatness = 25D0)

	integer key1, key2, key3, maxpass
	real*8 border, maxchisq, mindeviation
	integer ngiven, ldxgiven, nextra
	parameter (key1 = 47)
	parameter (key2 = 1)
	parameter (key3 = 1)
	parameter (maxpass = 5)
	parameter (border = 0D0)
	parameter (maxchisq = 10D0)
	parameter (mindeviation = .25D0)
	parameter (ngiven = 0)
	parameter (ldxgiven = ndim)
	parameter (nextra = 0)

	integer key
	parameter (key = 0)

	real*8 integral(ncomp), error(ncomp), prob(ncomp)
	integer verbose, nregions, neval, fail

********************************************************************
*                                                                  *      
***   End of variables for Integration                             *
*                                                                  *
********************************************************************
     
      real*8 rs,ma,mb,m1,m2,m3
      real*8 xrs,xma,xmb,xm1,xm2,xm3
      real*8 mw,mz,gw,gz,sw2,ee,GF,pi
      real*8 p3CMmax,E3max
      real*8 integrand,kallen
      real*8 xmin(10),xmax(10)
      real*8 xs(1)
      
      common/boundary/xmin,xmax
      common/data/mw,mz,gw,gz,sw2,ee,GF,pi
      common/parameters/xrs,xma,xmb,xm1,xm2,xm3
      
      external integrand
      verbose=0

      rs=xrs
      ma=xma
      mb=xmb
      m1=xm1
      m2=xm2
      m3=xm3
      
      p3CMmax=kallen(rs,m1+m2,m3)/2d0/rs
      E3max=sqrt(p3CMmax**2+m3**2)
      
      xmin(1)=m3
      xmax(1)=E3max
      xmin(2)=0.d0
      xmax(2)=pi
      xmin(3)=0.d0
      xmax(3)=pi
      xmin(4)=0.d0
      xmax(4)=2.d0*pi

      
       if(method.eq.1)then
         call gauss(ndim, ncomp, npoints, integrand, integral, fail)
         if(fail.eq.1)then
            write(*,*)'Gauss method only works for ndim <= 6'
            stop
         endif

         if(fail.eq.2)then
            write(*,*)'Gauss method only works for
     &           npoints=4,6,8,10,16,20,24,32,64,128'
            stop
         endif
         
      endif

      
      if(method.eq.2)then
	call vegas(ndim, ncomp, integrand, userdata, nvec,
     &    epsrel, epsabs, verbose, seed,
     &    mineval, maxeval, nstart, nincrease, nbatch,
     &    gridno, statefile, spin,
     &    neval, fail, integral, error, prob)
      endif


      xs(1)=integral(1)

      return
      end
*------------------------------------------------------------------------*      

*------------------------------------------------------------------------*
      subroutine DecayWidth(width,method,npoints)
      implicit none
      
********************************************************************
*                                                                  *      
***   Start variables for Integration in Cuba 4.2                  *
*                                                                  *
********************************************************************
*
***   Variables for Gauss
*

      integer npoints,method
*
***   Variables for all the methods
*
	integer ndim, ncomp, nvec, last, seed, mineval, maxeval
	real*8 epsrel, epsabs, userdata
	parameter (ndim = 4)
	parameter (ncomp = 1)
	parameter (userdata = 0)
	parameter (nvec = 1)
	parameter (epsrel = 1D-3)
	parameter (epsabs = 1D-12)
	parameter (last = 4)
	parameter (seed = 0)
	parameter (mineval = 0)
	parameter (maxeval = 50000)

	integer nstart, nincrease, nbatch, gridno
	integer*8 spin
	character*(*) statefile	
	parameter (nstart = 1000)
	parameter (nincrease = 500)
	parameter (nbatch = 1000)
	parameter (gridno = 0)
	parameter (statefile = "")
	parameter (spin = -1)

	integer nnew, nmin
	real*8 flatness
	parameter (nnew = 1000)
	parameter (nmin = 2)
	parameter (flatness = 25D0)

	integer key1, key2, key3, maxpass
	real*8 border, maxchisq, mindeviation
	integer ngiven, ldxgiven, nextra
	parameter (key1 = 47)
	parameter (key2 = 1)
	parameter (key3 = 1)
	parameter (maxpass = 5)
	parameter (border = 0D0)
	parameter (maxchisq = 10D0)
	parameter (mindeviation = .25D0)
	parameter (ngiven = 0)
	parameter (ldxgiven = ndim)
	parameter (nextra = 0)

	integer key
	parameter (key = 0)

	real*8 integral(ncomp), error(ncomp), prob(ncomp)
	integer verbose, nregions, neval, fail

********************************************************************
*                                                                  *      
***   End of variables for Integration                             *
*                                                                  *
********************************************************************
     
      real*8 rs,ma,mb,m1,m2,m3
      real*8 M0,xrs,xma,xmb,xm1,xm2,xm3
      real*8 mw,mz,gw,gz,sw2,ee,GF,pi
      real*8 p3CMmax,E3max
      real*8 integrand,kallen
      real*8 xmin(10),xmax(10)
      real*8 width
      
      common/boundary/xmin,xmax
      common/data/mw,mz,gw,gz,sw2,ee,GF,pi
      common/parameters/xrs,xma,xmb,xm1,xm2,xm3
      
      external integrand
      verbose=0
*
***   The value of the mass of decaying particle is passed as rs
***   for compatibility with the case of decays.      
*      
      M0=xrs
      m1=xm1
      m2=xm2
      m3=xm3

      p3CMmax=kallen(M0,m1+m2,m3)/2d0/M0
      E3max=sqrt(p3CMmax**2+m3**2)
      
      xmin(1)=m3
      xmax(1)=E3max
      xmin(2)=0.d0
      xmax(2)=pi
      xmin(3)=0.d0
      xmax(3)=pi
      xmin(4)=0.d0
      xmax(4)=2.d0*pi

      
       if(method.eq.1)then
         call gauss(ndim, ncomp, npoints, integrand, integral, fail)
         if(fail.eq.1)then
            write(*,*)'Gauss method only works for ndim <= 6'
            stop
         endif

         if(fail.eq.2)then
            write(*,*)'Gauss method only works for
     &           npoints=4,6,8,10,16,20,24,32,64,128'
            stop
         endif
         
      endif

      
      if(method.eq.2)then
	call vegas(ndim, ncomp, integrand, userdata, nvec,
     &    epsrel, epsabs, verbose, seed,
     &    mineval, maxeval, nstart, nincrease, nbatch,
     &    gridno, statefile, spin,
     &    neval, fail, integral, error, prob)
      endif


      width=integral(1)

      return
      end
*------------------------------------------------------------------------*      
      
*------------------------------------------------------------------------*
      subroutine integrand(ndim, xx, ncomp, ff)
      implicit none
      integer i, ndim, ncomp
      real*8 xx(*), ff(*)
      real*8 x(ndim),xmin(10),xmax(10),Jacob
      real*8 dsig

      common/boundary/xmin,xmax

      Jacob=1.d0

     
      do i=1,ndim
         Jacob=Jacob*(xmax(i)-xmin(i))
         x(i)=xmin(i)+(xmax(i)-xmin(i))*xx(i)
      enddo

      
      ff(1) = dsig(x)*Jacob

      return
      end
*------------------------------------------------------------------------*

*------------------------------------------------------------------------*
*     We call dsig either dsigma/dOmega or dGamma/dOmega                 *
*     The difference is the calling program                              *      
*------------------------------------------------------------------------*
      real*8 function dsig(x)
      implicit none
      real*8 theta,theta1
      real*8 rs,ma,mb,m1,m2,m3
      real*8 xrs,xma,xmb,xm1,xm2,xm3
      real*8 p1p2,p1q1,p1q2,p1q3,p2q1,p2q2,p2q3,q1q2,q1q3,q2q3,
     &     m12,p1p1,p2p2,q1q1,q2q2,q3q3,EPSp1p2q1q2
      real*8 M0,P0q1,P0q2,P0q3,P0P0

      
      real*8 factor,Flux,Msq,E3,p3CM,cst,kallen,Initial
      real*8 x(4)

      real*8 mw,mz,gw,gz,sw2,ee,GF,pi
      character*7 DecayOrXS

      common/data/mw,mz,gw,gz,sw2,ee,GF,pi
      common/parameters/xrs,xma,xmb,xm1,xm2,xm3
      common/Type/DecayOrXS

      
      if(DecayOrXS.eq."XSect23")then

         rs=xrs
         ma=xma
         mb=xmb
         m1=xm1
         m2=xm2
         m3=xm3
        
      call TBkinXS(x,rs,ma,mb,m1,m2,m3,p1p2,p1q1,p1q2,
     &     p1q3,p2q1,p2q2,p2q3,q1q2,q1q3,q2q3,m12,p1p1,p2p2,q1q1,
     &     q2q2,q3q3,EPSp1p2q1q2)

      E3=x(1)
      theta=x(2)
      theta1=x(3)
*
**** result in picobarn
*      
      factor=0.389379d09
      Flux= 4d0*sqrt(p1p2**2-ma**2*mb**2)
      p3CM=sqrt(E3**2-m3**2)

      
      cst=1d0/2d0/8d0*kallen(m12,m1,m2)/m12**2
      cst=cst*factor
      
      dsig=cst/Flux*sin(theta)*sin(theta1)*2d0*pi/(2d0*pi)**5*p3CM*
     &     Msq(rs,ma,mb,m1,m2,m3,p1p2,p1q1,p1q2,
     &     p1q3,p2q1,p2q2,p2q3,q1q2,q1q3,q2q3,m12,p1p1,p2p2,q1q1,
     &     q2q2,q3q3,EPSp1p2q1q2)

      elseif(DecayOrXS.eq."Decay13")then

         M0=xrs
         m1=xm1
         m2=xm2
         m3=xm3

      call TBkinDecays(x,M0,m1,m2,m3,P0q1,P0q2,P0q3,q1q2,q1q3,q2q3,
     &     m12,P0P0,q1q1,q2q2,q3q3)

      E3=x(1)
      theta=x(2)
      theta1=x(3)
*
**** result in GeV
*      
      factor=1d0
      Initial= 2d0*M0
      p3CM=sqrt(E3**2-m3**2)

      cst=1d0/2d0/8d0*kallen(m12,m1,m2)/m12**2
      cst=cst*factor
      
      dsig=cst/Initial*sin(theta)*sin(theta1)*2d0*pi/(2d0*pi)**5*p3CM*
     &     Msq(M0,m1,m2,m3,P0q1,P0q2,P0q3,q1q2,q1q3,q2q3,
     &     m12,P0P0,q1q1,q2q2,q3q3)

      else
          dsig=0d0
      endif
      
      return
      end
*------------------------------------------------------------------------*
*------------------------------------------------------------------------*
      subroutine TBkinXS(x,rs,ma,mb,m1,m2,m3,p1p2,p1q1,p1q2,
     &     p1q3,p2q1,p2q2,p2q3,q1q2,q1q3,q2q3,m12,p1p1,p2p2,q1q1,
     &     q2q2,q3q3,EPSp1p2q1q2)
      implicit none
      
      real*8 x(4),E3,teta,teta1,phi1
      real*8 s,p1CM,q3CM,m12,q12CM,p3CM
      real*8 rs,ma,mb,m1,m2,m3,p1p2,p1q1,p1q2,p1q3,p2q1,p2q2,p2q3,
     &     q1q2,q1q3,q2q3,EPSp1p2q1q2
      real*8 p1p1,p2p2,q1q1,q2q2,q3q3
      real*8 p1(4),p2(4),q1(4),q2(4),q3(4)
      real*8 st,ct,st1,ct1,sp1,cp1
      real*8 xb,xg,dot,beta,gama,kallen,get_m12,epsilon

      E3=x(1)
      teta=x(2)
      teta1=x(3)
      phi1=x(4)

      s=rs**2
      
      p1CM=kallen(rs,ma,mb)/2d0/rs
      
      p1(4)=(s + ma**2 - mb**2)/2d0/rs
      p1(1)=0.d0
      p1(2)=0.d0
      p1(3)=p1CM

      
      p2(4)=(s + mb**2 - ma**2)/2d0/rs
      p2(1)=0.d0
      p2(2)=0.d0
      p2(3)=-p1CM
      
      m12=get_m12(rs,m3,E3)
      p3CM=kallen(rs,m12,m3)/2d0/rs
      xg=gama(rs,m3,E3)
      xb=beta(rs,m3,E3)

      ct=cos(teta)
      st=sin(teta)
      ct1=cos(teta1)
      st1=sin(teta1)
      sp1=sin(phi1)
      cp1=cos(phi1)

      q3CM=sqrt(E3**2-m3**2)
      q12CM=kallen(m12,m1,m2)/2d0/m12
      
      q1(4)=(m12**2+m1**2-m2**2)/(2d0*m12)*xg + q12CM*xb*xg*ct1
      q1(1)=q12CM*(-ct*st1*cp1-st*xg*ct1) -
     &     (m12**2+m1**2-m2**2)/(2d0*m12)*xb*xg*st
      q1(2)=q12CM*st1*sp1
      q1(3)=q12CM*(st*st1*cp1-ct*xg*ct1) -
     &     (m12**2+m1**2-m2**2)/(2d0*m12)*xb*xg*ct
      
      q2(4)=(m12**2+m2**2-m1**2)/(2d0*m12)*xg - q12CM*xb*xg*ct1
      q2(1)=q12CM*(ct*st1*cp1+ct1*st*xg) -
     &     (m12**2+m2**2-m1**2)/(2d0*m12)*xb*xg*st
      q2(2)=q12CM*(-st1*sp1)
      q2(3)=q12CM*(-cp1*st1*st+ct1*ct*xg) -
     &     (m12**2+m2**2-m1**2)/(2d0*m12)*xb*xg*ct      
      
      q3(4)=E3
      q3(1)=q3CM*st
      q3(2)=0d0
      q3(3)=q3CM*ct

      p1p1=dot(p1,p1)
      p2p2=dot(p2,p2)
      q1q1=dot(q1,q1)
      q2q2=dot(q2,q2)
      q3q3=dot(q3,q3)
      
      p1p2=dot(p1,p2)
      p1q1=dot(p1,q1)
      p1q2=dot(p1,q2)
      p1q3=dot(p1,q3)
      p2q1=dot(p2,q1)
      p2q2=dot(p2,q2)
      p2q3=dot(p2,q3)
      q1q2=dot(q1,q2)
      q1q3=dot(q1,q3)
      q2q3=dot(q2,q3)

      EPSp1p2q1q2=epsilon(p1,p2,q1,q2)
      
      return
      end
*------------------------------------------------------------------------*
*------------------------------------------------------------------------*
      subroutine TBkinDecays(x,M0,m1,m2,m3,P0q1,P0q2,P0q3,q1q2,q1q3,
     &     q2q3,m12,P0P0,q1q1,q2q2,q3q3)


      implicit none
      
      real*8 x(4),E3,teta,teta1,phi1
      real*8 s,q3CM,q12CM,p3CM
      real*8 M0,m1,m2,m3,P0q1,P0q2,P0q3,q1q2,q1q3,q2q3,
     &     m12,P0P0,q1q1,q2q2,q3q3

      real*8 P0(4),q1(4),q2(4),q3(4)
      real*8 st,ct,st1,ct1,sp1,cp1
      real*8 xb,xg,dot,beta,gama,kallen,get_m12,epsilon

      E3=x(1)
      teta=x(2)
      teta1=x(3)
      phi1=x(4)

      s=M0**2
      
      
      P0(4)=M0
      P0(1)=0.d0
      P0(2)=0.d0
      P0(3)=0d0

      m12=get_m12(M0,m3,E3)
      p3CM=kallen(M0,m12,m3)/2d0/M0
      xg=gama(M0,m3,E3)
      xb=beta(M0,m3,E3)

      ct=cos(teta)
      st=sin(teta)
      ct1=cos(teta1)
      st1=sin(teta1)
      sp1=sin(phi1)
      cp1=cos(phi1)

      q3CM=sqrt(E3**2-m3**2)
      q12CM=kallen(m12,m1,m2)/2d0/m12
      
      q1(4)=(m12**2+m1**2-m2**2)/(2d0*m12)*xg + q12CM*xb*xg*ct1
      q1(1)=q12CM*(-ct*st1*cp1-st*xg*ct1) -
     &     (m12**2+m1**2-m2**2)/(2d0*m12)*xb*xg*st
      q1(2)=q12CM*st1*sp1
      q1(3)=q12CM*(st*st1*cp1-ct*xg*ct1) -
     &     (m12**2+m1**2-m2**2)/(2d0*m12)*xb*xg*ct
      
      q2(4)=(m12**2+m2**2-m1**2)/(2d0*m12)*xg - q12CM*xb*xg*ct1
      q2(1)=q12CM*(ct*st1*cp1+ct1*st*xg) -
     &     (m12**2+m2**2-m1**2)/(2d0*m12)*xb*xg*st
      q2(2)=q12CM*(-st1*sp1)
      q2(3)=q12CM*(-cp1*st1*st+ct1*ct*xg) -
     &     (m12**2+m2**2-m1**2)/(2d0*m12)*xb*xg*ct      
      
      q3(4)=E3
      q3(1)=q3CM*st
      q3(2)=0d0
      q3(3)=q3CM*ct

      P0P0=M0**2
      q1q1=dot(q1,q1)
      q2q2=dot(q2,q2)
      q3q3=dot(q3,q3)
      
      P0q1=dot(P0,q1)
      P0q2=dot(P0,q2)
      P0q3=dot(P0,q3)
      q1q2=dot(q1,q2)
      q1q3=dot(q1,q3)
      q2q3=dot(q2,q3)

      return
      end
*------------------------------------------------------------------------*      
      
*------------------------------------------------------------------------*
      double precision function kallen(x,y,z)
      real*8 x,y,z
      kallen=sqrt(x**4 + y**4 + z**4 -2d0*x**2*y**2 - 2d0*x**2*z**2
     &     -2d0*y**2*z**2)
      return
      end
      
      double precision function get_m12(rs,m3,E3)
      implicit none
      double precision rs,m3,E3
      get_m12=sqrt(rs**2+m3**2-2.d0*rs*E3)
      return
      end

      double precision function e12(rs,m3,E3)
      implicit none
      double precision E3,get_m12,rs,m3
      e12=(rs**2+get_m12(rs,m3,e3)**2-m3**2)/2.d0/rs
      return
      end

      double precision function gama(rs,m3,E3)
      implicit none
      double precision e12,get_m12,E3,rs,m3
      gama=e12(rs,m3,E3)/get_m12(rs,m3,E3)
      return
      end

      double precision function beta(rs,m3,E3)
      implicit none
      double precision gama,E3,rs,m3
      beta=sqrt(1.d0-1.d0/gama(rs,m3,E3)**2)
      return
      end

      double precision function dot(p,q)
      implicit none
      double precision p(4),q(4)
      dot=p(4)*q(4)-p(1)*q(1)-p(2)*q(2)-p(3)*q(3)
      return
      end

      real*8 function epsilon(p1,p2,p3,p4)
      implicit none
      real*8 p1(4),p2(4),p3(4),p4(4)
      epsilon=(p1(4)*p2(3)-p1(3)*p2(4))*(p3(1)*p4(2)-p3(2)*p4(1))

      epsilon=-epsilon

      return
      end
*------------------------------------------------------------------------*      
*------------------------------------------------------------------------*      
      subroutine gauss(ndim, ncomp, npoints, integrand, integral, fail)
      implicit none
      integer n,ndim, npoints,i1,i2,i3,i4,i5,i6,ncomp,fail
      double precision integral(*),x(npoints),w(npoints)
      double precision xx(ndim), ff(ncomp)


      fail=0

      call dsetgauss(0.d0,1.d0,npoints,x,w,fail)

      if(ndim.eq.1)then

         do n=1,ncomp
            integral(n)=0.d0
            do i1=1,npoints
               xx(1)=x(i1)
               call integrand(ndim, xx, ncomp, ff)
               integral(n)=integral(n) + ff(n)*w(i1)
            enddo
         enddo
         return

      else if(ndim.eq.2)then
         
         do n=1,ncomp
            integral(n)=0.d0
            do i1=1,npoints
               xx(1)=x(i1)
               do i2=1,npoints
                  xx(2)=x(i2)
                  call integrand(ndim, xx, ncomp, ff)
                  integral(n)=integral(n) + ff(n)*w(i1)*w(i2)
               enddo
            enddo
         enddo
         return


      else if(ndim.eq.3)then
         
         do n=1,ncomp
            integral(n)=0.d0
            do i1=1,npoints
               xx(1)=x(i1)
               do i2=1,npoints
                  xx(2)=x(i2)
                  do i3=1,npoints
                     xx(3)=x(i3)
                     call integrand(ndim, xx, ncomp, ff)
                     integral(n)=integral(n) + ff(n)*w(i1)*w(i2)*w(i3)
                  enddo
               enddo
            enddo
         enddo
         return

      else if(ndim.eq.4)then
         
         do n=1,ncomp
            integral(n)=0.d0
            do i1=1,npoints
               xx(1)=x(i1)
               do i2=1,npoints
                  xx(2)=x(i2)
                  do i3=1,npoints
                     xx(3)=x(i3)
                     do i4=1,npoints
                        xx(4)=x(i4)
                        call integrand(ndim, xx, ncomp, ff)
                        integral(n)=integral(n) 
     &                       + ff(n)*w(i1)*w(i2)*w(i3)*w(i4)
                     enddo
                  enddo
               enddo
            enddo
         enddo
         return

      else if(ndim.eq.5)then
         
         do n=1,ncomp
            integral(n)=0.d0
            do i1=1,npoints
               xx(1)=x(i1)
               do i2=1,npoints
                  xx(2)=x(i2)
                  do i3=1,npoints
                     xx(3)=x(i3)
                     do i4=1,npoints
                        xx(4)=x(i4)
                        do i5=1,npoints
                           xx(5)=x(i5)
                           call integrand(ndim, xx, ncomp, ff)
                           integral(n)=integral(n) 
     &                          + ff(n)*w(i1)*w(i2)*w(i3)*w(i4)*w(i5)
                        enddo
                     enddo
                  enddo
               enddo
            enddo
         enddo
         return

      else if(ndim.eq.6)then
         
         do n=1,ncomp
            integral(n)=0.d0
            do i1=1,npoints
               xx(1)=x(i1)
               do i2=1,npoints
                  xx(2)=x(i2)
                  do i3=1,npoints
                     xx(3)=x(i3)
                     do i4=1,npoints
                        xx(4)=x(i4)
                        do i5=1,npoints
                           xx(5)=x(i5)
                           do i6=1,npoints
                              xx(6)=x(i6)
                              call integrand(ndim, xx, ncomp, ff)
                              integral(n)=integral(n) 
     &                             + ff(n)*w(i1)*w(i2)*w(i3)*w(i4)
     &                             *w(i5)*w(i6)
                           enddo
                        enddo
                     enddo
                  enddo
               enddo
            enddo
         enddo
         return

      else
         fail=1
         return
      endif


      end



c
c       This subroutine stores the values for
c       the gaussian integration
c       Only works with n=4,6,8,10,16,20,24,32,64,128
c
      subroutine dsetgauss(a,b,n,x,w,fail)
      implicit none
      integer i,n,nm,kiplus,kiminus,fail
      real*8 a,b,gauss4(4),gauss6(6),gauss8(8),work(256)
      real*8 gauss10(10),gauss16(16),gauss20(20)
      real*8 gauss24(24),gauss32(32),gauss64(64),gauss128(128)
      real*8 x(128),w(128),xuni(128),wuni(128)
      data gauss4/0.339981043584856d0,0.652145154862546d0,  
     #0.861136311594053d0,0.347854845137454d0/
      data gauss6/0.238619186083197d0,0.467913934572691,
     #0.661209386466265d0,0.360761573048139,
     #0.932469514203152d0,0.171324492379170d0/
      data gauss8/0.183434642495650d0,0.362683783378362d0,
     #0.525532409916329d0,0.313706645877887d0,
     #0.796666477413627d0,0.222381034453374d0,
     #0.960289856497536d0,0.101228536290376d0/
      data gauss10/0.148874338981631d0,0.295524224714753d0,
     #0.433395394129247d0,0.269266719309996d0,
     #0.679409568299024d0,0.219086362515982d0,
     #0.865063366688985d0,0.149451349150581d0,
     #0.973906528517172d0,0.066671344308688d0/
      data gauss16/0.095012509837637d0,0.189450610455068d0,
     #0.281603550779258d0,0.182603415044923d0,
     #0.458016777657227d0,0.169156519395002d0,
     #0.617876244402643d0,0.149595988816576d0,
     #0.755404408355003d0,0.124628971255533d0,
     #0.865631202387831d0,0.095158511682492d0,
     #0.944575023073232d0,0.062253523938647d0,
     #0.989400934991649d0,0.027152459411754d0/
      data gauss20/0.076526521133497d0,0.152753387130725d0,
     #0.227785851141645d0,0.149172986472603d0,
     #0.373706088715419d0,0.142096109318382d0,
     #0.510867001950827d0,0.131688638449176d0,
     #0.636053680726515d0,0.118194531961518d0,
     #0.746331906460150d0,0.101930119817240d0,
     #0.839116971822218d0,0.083276741576704d0,
     #0.912234428251325d0,0.062672048334109d0,
     #0.963971927277913d0,0.040601429800386d0,
     #0.993128599185094d0,0.017614007139152d0/
      data gauss24/0.064056892862605d0,0.127938195346752d0,
     #0.191118867473616d0,0.125837456346828d0,
     #0.315042679696163d0,0.121670472927803d0,
     #0.433793507626045d0,0.115505668053725d0,
     #0.545421471388839d0,0.107444270115965d0,
     #0.648093651936975d0,0.097618652104113d0,
     #0.740124191578554d0,0.086190161531953d0,
     #0.820001985973902d0,0.073346481411080d0,
     #0.886415527004401d0,0.059298584915436d0,
     #0.938274552002732d0,0.044277438817419d0,
     #0.974728555971309d0,0.028531388628933d0,
     #0.995187219997021d0,0.012341229799987d0/
      data gauss32/0.048307665687738d0,0.096540088514727d0,
     #0.144471961582796d0,0.095638720079274d0,
     #0.239287362252137d0,0.093844399080804d0,
     #0.331868602282127d0,0.091173878695763d0,
     #0.421351276130635d0,0.087652093004403d0,
     #0.506899908932229d0,0.083311924226946d0,
     #0.587715757240762d0,0.078193895787070d0,
     #0.663044266930215d0,0.072345794108848d0,
     #0.732182118740289d0,0.065822222776361d0,
     #0.794483795967942d0,0.058684093478535d0,
     #0.849367613732569d0,0.050998059262376d0,
     #0.896321155766052d0,0.042835898022226d0,
     #0.934906075937739d0,0.034273862913021d0,
     #0.964762255587506d0,0.025392065309262d0,
     #0.985611511545268d0,0.016274394730905d0,
     #0.997263861849481d0,0.007018610009470d0/
      data gauss64/0.02435029266342d0,0.04869095700914d0,
     &0.07299312178780d0,0.04857546744150d0,
     &0.12146281929612d0,0.04834476223477d0,
     &0.16964442042399d0,0.04799938859631d0,
     &0.21742364374001d0,0.04754016571443d0,
     &0.26468716220877d0,0.04696818281529d0,
     &0.31132287199021d0,0.04628479657947d0,
     &0.35722015833767d0,0.04549162792405d0,
     &0.40227015796399d0,0.04459055815801d0,
     &0.44636601725346d0,0.04358372451999d0,
     &0.48940314570705d0,0.04247351510908d0,
     &0.53127946401989d0,0.04126256322055d0,
     &0.57189564620263d0,0.03995374110007d0,
     &0.61115535517239d0,0.03855015317862d0,
     &0.64896547125466d0,0.03705512854024d0,
     &0.68523631305423d0,0.03547221325688d0,
     &0.71988185017161d0,0.03380516183714d0,
     &0.75281990726053d0,0.03205792835485d0,
     &0.78397235894334d0,0.03023465707240d0,
     &0.81326531512280d0,0.02833967261426d0,
     &0.84062929625258d0,0.02637746971505d0,
     &0.86599939815409d0,0.02435270256871d0,
     &0.88931544599511d0,0.02227017380838d0,
     &0.91052213707850d0,0.02013482315353d0,
     &0.92956917213194d0,0.01795171577570d0,
     &0.94641137485840d0,0.01572603047602d0,
     &0.96100879965205d0,0.01346304789672d0,
     &0.97332682778991d0,0.01116813946013d0,
     &0.98333625388463d0,0.00884675982634d0,
     &0.99101337147674d0,0.00650445796881d0,
     &0.99634011677196d0,0.00414703325659d0,
     &0.99930504173577d0,0.00178327963660d0/
      data gauss128/0.01222369896062d0,0.02444618019626d0,
     &0.03666379096873d0,0.02443156909785d0,
     &0.06108196960414d0,0.02440235563385d0,
     &0.08546364050452d0,0.02435855726469d0,
     &0.10979423112764d0,0.02430020016797d0,
     &0.13405919946119d0,0.02422731922281d0,
     &0.15824404271422d0,0.02413995798902d0,
     &0.18233430598534d0,0.02403816868102d0,
     &0.20631559090208d0,0.02392201213669d0,
     &0.23017356422666d0,0.02379155778099d0,
     &0.25389396642269d0,0.02364688358442d0,
     &0.27746262017790d0,0.02348807601650d0,
     &0.30086543887768d0,0.02331522999401d0,
     &0.32408843502441d0,0.02312844882432d0,
     &0.34711772859764d0,0.02292784414359d0,
     &0.36993955534986d0,0.02271353585011d0,
     &0.39254027503327d0,0.02248565203258d0,
     &0.41490637955227d0,0.02224432889359d0,
     &0.43702450103710d0,0.02198971066819d0,
     &0.45888141983355d0,0.02172194953771d0,
     &0.48046407240417d0,0.02144120553878d0,
     &0.50175955913614d0,0.02114764646770d0,
     &0.52275515205118d0,0.02084144778010d0,
     &0.54343830241281d0,0.02052279248617d0,
     &0.56379664822662d0,0.02019187104117d0,
     &0.58381802162876d0,0.01984888123167d0,
     &0.60349045615855d0,0.01949402805730d0,
     &0.62280219391058d0,0.01912752360826d0,
     &0.64174169256231d0,0.01874958693853d0,
     &0.66029763227265d0,0.01836044393492d0,
     &0.67845892244772d0,0.01796032718214d0,
     &0.69621470836951d0,0.01754947582371d0,
     &0.71355437768359d0,0.01712813541906d0,
     &0.73046756674191d0,0.01669655779679d0,
     &0.74694416679706d0,0.01625500090410d0,
     &0.76297433004409d0,0.01580372865266d0,
     &0.77854847550641d0,0.01534301076088d0,
     &0.79365729476219d0,0.01487312259269d0,
     &0.80829175750791d0,0.01439434499294d0,
     &0.82244311695564d0,0.01390696411961d0,
     &0.83610291506091d0,0.01341127127272d0,
     &0.84926298757797d0,0.01290756272029d0,
     &0.86191546893955d0,0.01239613952122d0,
     &0.87405279695803d0,0.01187730734541d0,
     &0.88566771734540d0,0.01135137629108d0,
     &0.89675328804916d0,0.01081866069944d0,
     &0.90730288340176d0,0.01027947896690d0,
     &0.91731019808096d0,0.00973415335479d0,
     &0.92676925087895d0,0.00918300987166d0,
     &0.93567438827792d0,0.00862637779862d0,
     &0.94402028783022d0,0.00806458989049d0,
     &0.95180196134126d0,0.00749798192563d0,
     &0.95901475785370d0,0.00692689256690d0,
     &0.96565436643197d0,0.00635166316171d0,
     &0.97171681874714d0,0.00577263754287d0,
     &0.97719849146391d0,0.00519016183268d0,
     &0.98209610843572d0,0.00460458425670d0,
     &0.98640674272459d0,0.00401625498374d0,
     &0.99012781849173d0,0.00342552604091d0,
     &0.99325711290021d0,0.00283275147146d0,
     &0.99579275853498d0,0.00223828843096d0,
     &0.99773324862551d0,0.00164250301862d0,
     &0.99907745997738d0,0.00104581267832d0,
     &0.99982488794713d0,0.00044938068609d0/


      nm=n/2

      if(n.eq.4)then
         do i=1,n
            work(i)=gauss4(i)
         enddo
      else if(n.eq.6)then
         do i=1,n
            work(i)=gauss6(i)
         enddo
      else if(n.eq.8)then
         do i=1,n
            work(i)=gauss8(i)
         enddo
      else if(n.eq.10)then
         do i=1,n
            work(i)=gauss10(i)
         enddo
      else if(n.eq.16)then
         do i=1,n
            work(i)=gauss16(i)
         enddo
      else if(n.eq.20)then
         do i=1,n
            work(i)=gauss20(i)
         enddo
      else if(n.eq.24)then
         do i=1,n
            work(i)=gauss24(i)
         enddo
      else if(n.eq.32)then
         do i=1,n
            work(i)=gauss32(i)
         enddo
      else if(n.eq.64)then
         do i=1,n
            work(i)=gauss64(i)
         enddo
      else if(n.eq.128)then
         do i=1,n
            work(i)=gauss128(i)
         enddo
      else
         fail=2
         return
      endif

      

      do 10 i=1,nm
      kiplus=nm+i
      kiminus=nm+1-i
      xuni(kiplus)=work(2*i-1)
      xuni(kiminus)=-work(2*i-1)
      wuni(kiplus)=work(2*i)
      wuni(kiminus)=work(2*i)
 10   continue

      do i=1,n
      x(i)=(b-a)/2.d0*xuni(i)+(b+a)/2.
      w(i)=(b-a)/2.d0*wuni(i)
      end do

      return
      end






